﻿using System;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Widgets;

/// <summary>
/// Vihollisen luokka
/// </summary>
public class Vihollinen : PhysicsObject
{
    private int osumat;
    private int elama = 2;
    public Vihollinen(double leveys, double korkeus)
        : base(leveys, korkeus)
    {
        osumat = 0;

    }

    /// <summary>
    /// Kutsutaan kun viholliseen osuu panos. Vihollisella on 3 elämää
    /// </summary>
    public void OtaVastaanOsuma()
    {
        
        osumat++;
        if (osumat > elama)
        {
            this.Destroy();
        }
    }
}

///@author Teemu Säily
///@version 23.4.2020
/// <summary>
/// 
/// </summary>
public class Harjoitustyo : PhysicsGame
{
    private const int KOKO = 80;
    private const int PANOKSEN_KOKO = 20;
    private IntMeter pisteLaskuri;
    private Vector[] vihuSpawnit = new Vector[5];
    private Vector[] tykkiSpawnit = new Vector[20];
    private int vihuTaulukkoPaikka = 0;
    private int tykkiTaulukkoPaikka = 0;
    private int tykkiLuku = 0;
    private int elamat = 3;
   
    
    /// <summary>
    /// Kutsutaan kun peli alkaa
    /// </summary>
    public override void Begin()
    {
        TeeKentta();
        MultiSelectWindow alkuValikko = new MultiSelectWindow("Pelin alkuvalikko", "Aloita peli", "Lopeta");
        Add(alkuValikko);
        alkuValikko.AddItemHandler(0, AloitaPeli);
        alkuValikko.AddItemHandler(1, Exit);

        Mouse.Listen(MouseButton.Left, ButtonState.Released, LisaaTykki, "Lisää peliin tykin");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
        Keyboard.Listen(Key.P, ButtonState.Pressed, Pause, "Pausettaa pelin");
    }


    /// <summary>
    /// Kutsutaan kun peli alkaa. Kutsutaan pelin aloittavia aliohjelmia
    /// </summary>
    private void AloitaPeli()
    {
        AloitaVihuAjastin();
        LisaaVihollisia();
        OnkoTykkiLupa();
    }


    /// <summary>
    /// Kutsutaan, kun elämät loppuvat ja peli loppuu
    /// </summary>
    private void PelinLopetus()
    {
        ClearGameObjects();
        Pause();
        Label lopetusNaytto = new Label(300, 300, "Pisteesi " + pisteLaskuri.ToString());
        Add(lopetusNaytto);
    }


    /// <summary>
    /// Pelin "raha". Ilman rahaa tykkiä ei voi ostaa
    /// </summary>
    private void OnkoTykkiLupa()
    {
        Timer tykkiLupa = new Timer();
        tykkiLupa.Interval = 3.0;
        tykkiLupa.Timeout += TykkiLuku;
        tykkiLupa.Start();
    }


    /// <summary>
    /// Lisää kutsuttaessa rahan määrää
    /// </summary>
    private void TykkiLuku()
    {
        tykkiLuku++;
    }


    /// <summary>
    /// Lisää peliin tykin. Jos ei ole rahaa tykkiin, printtaa näytölle tekstiä
    /// </summary>
    private void LisaaTykki()
    {
        if (tykkiLuku > 0)
        {
            PhysicsObject tykki = new PhysicsObject(KOKO, KOKO, Shape.Rectangle);
            tykki.Position = LahinTykkiSpawn();
            tykki.Color = Color.ForestGreen;
            tykki.Image = LoadImage("tykki");
            tykki.CollisionIgnoreGroup = 2;
            tykki.MakeStatic();
            Add(tykki);
            AmmuTykillä(tykki);
            TykkiAjastin(tykki);
            AddCollisionHandler<PhysicsObject, Vihollinen>(tykki, "vihu", CollisionHandler.DestroyBoth);
            tykkiLuku--;
        }
        else
        {
            Label tykkiLukuNaytto = new Label();
            tykkiLukuNaytto.Position = new Vector(0,-300);
            tykkiLukuNaytto.TextColor = Color.Black;
            tykkiLukuNaytto.Color = Color.White;
            tykkiLukuNaytto.Text = "Ei vapaita tykkejä";
            tykkiLukuNaytto.LifetimeLeft = TimeSpan.FromSeconds(2);
            Add(tykkiLukuNaytto);
        }
    }


    /// <summary>
    /// Hoitaa tykillä ampumisen eli luo tykistä panoksia
    /// </summary>
    /// <param name="tykki">Ampuva tykki</param>
    private void AmmuTykillä(PhysicsObject tykki)
    {
        PhysicsObject panos = new PhysicsObject(PANOKSEN_KOKO, PANOKSEN_KOKO);
        panos.IgnoresGravity = true;
        panos.Shape = Shape.Circle;
        panos.Color = Color.BloodRed;
        panos.CollisionIgnoreGroup = 2;
        panos.Position = tykki.Position;
        panos.LinearDamping = 1;
        panos.Velocity = new Vector(1000, 0);
        Add(panos);
        AddCollisionHandler<PhysicsObject, Vihollinen>(panos, "vihu", TormattiinViholliseen);
        //AddCollisionHandler(panos, "vihu", CollisionHandler.ExplodeObject(panoksenKoko, true));
        panos.LifetimeLeft = TimeSpan.FromSeconds(1.5);
    }
    

    /// <summary>
    /// Ajastin tykin automaattiselle ampumiselle
    /// </summary>
    /// <param name="tykki">Ampuva tykki</param>
    private void TykkiAjastin(PhysicsObject tykki)
    {
        Timer tykkiAjastin = new Timer();
        tykkiAjastin.Interval = 1.0;
        tykkiAjastin.Timeout += delegate { AmmuTykillä(tykki); }; 
        tykkiAjastin.Start();
        tykki.Destroyed += tykkiAjastin.Stop;
    }


    /// <summary>
    /// Kutsutaan kun panos osuu viholliseen. Vähentää viholliselta yhden elämäpisteen 
    /// </summary>
    /// <param name="panos">Osuva panos</param>
    /// <param name="kohde">Osuva vihollinen</param>
    private void TormattiinViholliseen(PhysicsObject panos, Vihollinen kohde)
    {
        kohde.OtaVastaanOsuma();
        panos.Destroy();
    }


    /// <summary>
    /// Etsii hiirtä läheisimpää tykkispawnia, johon tykin voi tehdä
    /// </summary>
    /// <returns>Hiirtä lähimmän tykkipaikan</returns>
    private Vector LahinTykkiSpawn()
    {
        Vector lahin = tykkiSpawnit[0];
        foreach (Vector spawn in tykkiSpawnit)
        {
            if (Vector.Distance(Mouse.PositionOnWorld, spawn) < Vector.Distance(Mouse.PositionOnWorld, lahin))
            {
                lahin = spawn;
            }
        }
        return lahin;
    }


    /// <summary>
    /// Tekee peliin kentän tekstitiedostosta
    /// </summary>
    private void TeeKentta()
    {
        TileMap kentta = TileMap.FromLevelAsset("Leveli");
        kentta.SetTileMethod('l', LuoLattia, "lattia");
        kentta.SetTileMethod('v', LuoSpawni, "vihujen spawnpisteet", 'v');
        kentta.SetTileMethod('p', LuoSpawni, "tykkien spawnipisteet", 'p');
        kentta.SetTileMethod('k', LuoPisteLaskuri, "pistelaskuri");
        kentta.SetTileMethod('s', LuoTakaSeina, "takaseinä");
        kentta.Execute(KOKO, KOKO);
        kentta.Optimize('l');
        Level.Size = new Vector(1000, 800);
        Level.Background.Image = LoadImage("taustakuva");
        SetWindowSize(1000, 800);
        LuoElamaLaskuri();
        Camera.ZoomToAllObjects();
        Gravity = new Vector(0, -400);
    }


    /// <summary>
    /// Kutsuttaessa luo peliin lattian
    /// </summary>
    /// <param name="paikka">Lattian paikka</param>
    /// <param name="korkeus">Lattian korkeus</param>
    /// <param name="leveys">Lattian leveys</param>
    /// <param name="kuvanNimi">Lattian haluttu kuva</param>
    private void LuoLattia(Vector paikka, double korkeus, double leveys, string kuvanNimi)
    {
        PhysicsObject lattia = new PhysicsObject(KOKO, KOKO);
        lattia.Position = paikka;
        lattia.Image = LoadImage("tiili");
        lattia.MakeStatic();
        lattia.StaticFriction = 0.1;
        lattia.KineticFriction = 0.1;
        lattia.CollisionIgnoreGroup = 1;
        Add(lattia);
    }


    /// <summary>
    /// Luo seinän, johon viholliset pyrkivät
    /// </summary>
    /// <param name="paikka">Seinän paikka</param>
    /// <param name="korkeus">Seinän korkeus</param>
    /// <param name="leveys">Seinän leveys</param>
    /// <param name="kuvanNimi">Seinän kuva</param>
    private void LuoTakaSeina(Vector paikka, double korkeus, double leveys, string kuvanNimi)
    {
        PhysicsObject takaSeina = new PhysicsObject(KOKO, KOKO);
        takaSeina.Position = paikka;
        takaSeina.Color = Color.Blue;
        takaSeina.MakeStatic();
        takaSeina.CollisionIgnoreGroup = 1;
        Add(takaSeina);
        AddCollisionHandler<PhysicsObject, Vihollinen>(takaSeina, "vihu", VahennaElama);
        AddCollisionHandler<PhysicsObject, Vihollinen>(takaSeina, "vihu", CollisionHandler.DestroyTarget);
    }


    /// <summary>
    /// Vähentää pelaajalta elämän. Kutsutaan kun vihollinen osuu takaseinään. Kutsuu pelin lopetusta jos elämät ovat loppu
    /// </summary>
    /// <param name="seina">Takaseinä</param>
    /// <param name="kohde">Viholinen</param>
    private void VahennaElama(PhysicsObject seina, Vihollinen kohde)
    {
        if (elamat > 0)
        {
            elamat--;
            Game.DoNextUpdate(LuoElamaLaskuri);
        }
        else PelinLopetus();
    }


    /// <summary>
    /// Luo tekstitiedostosta paikat vihollisille ja tykeille
    /// </summary>
    /// <param name="paikka">Paikka</param>
    /// <param name="korkeus">Korkeus</param>
    /// <param name="leveys">Leveys</param>
    /// <param name="kuvanNimi">Paikan kuva</param>
    /// <param name="tunniste">Viholliselle vai tykille</param>
    private void LuoSpawni(Vector paikka, double korkeus, double leveys, string kuvanNimi, char tunniste)
    {
        PhysicsObject spawn = new PhysicsObject(KOKO, KOKO);
        spawn.IgnoresCollisionResponse = true;
        spawn.IgnoresGravity = true;
        spawn.Position = paikka;
        spawn.IsVisible = false;
        
        if (tunniste == 'v')
        {
            vihuSpawnit[vihuTaulukkoPaikka] = spawn.Position;
            vihuTaulukkoPaikka++;
        }
        else
        {
            tykkiSpawnit[tykkiTaulukkoPaikka] = spawn.Position;
            tykkiTaulukkoPaikka++;
        }
        Add(spawn);
    }


    /// <summary>
    /// Luo peliin laskurin joka laskee pisteitä
    /// </summary>
    /// <param name="paikka">Laskurin paikka</param>
    /// <param name="leveys">Leveys</param>
    /// <param name="korkeus">Korkeus</param>
    /// <param name="kuvanNimi">Laskurin kuva</param>
    private void LuoPisteLaskuri(Vector paikka, double leveys, double korkeus, string kuvanNimi)
    {
        pisteLaskuri = new IntMeter(0);

        Label pisteNaytto = new Label();
        pisteNaytto.Position = new Vector(200,350);
        pisteNaytto.TextColor = Color.Black;
        pisteNaytto.Color = Color.White;
        pisteNaytto.Text = "Pisteet" + pisteLaskuri;
        pisteNaytto.BindTo(pisteLaskuri);
        Add(pisteNaytto);
    }


    /// <summary>
    /// Luo peliin laskurin joka laskee elämiä
    /// </summary>
    private void LuoElamaLaskuri()
    {
        Label elamaNaytto = new Label();
        elamaNaytto.Position = new Vector(0, 350);
        elamaNaytto.TextColor = Color.Black;
        elamaNaytto.Color = Color.White;
        elamaNaytto.Text = "Elämät " + elamat.ToString();
        Add(elamaNaytto);
        elamaNaytto.IsUpdated = true;
    }


    /// <summary>
    /// Kutsutaan "aloita peli" -nappia painettaessa, aloittaa vihollisten spawnaamisen
    /// </summary>
    private void AloitaVihuAjastin()
    {
        Timer vihuAjastin = new Timer();
        vihuAjastin.Interval = 4.0;
        vihuAjastin.Timeout += LuoVihu;
        vihuAjastin.Start();
    }


    /// <summary>
    /// Pelin vaikeustasosäädin. Ajan myötä lisää vihollisten spawnnopeutta
    /// </summary>
    private void LisaaVihollisia()
    {
        Timer vihuAjastin = new Timer();
        vihuAjastin.Interval = 10.0;
        vihuAjastin.Timeout += AloitaVihuAjastin;
        vihuAjastin.Start();
    }
    

    /// <summary>
    /// Kutsuttaessa luo vihollisen johonkin vihollisspawniin
    /// </summary>
    private void LuoVihu()
    {
        Vihollinen vihu = new Vihollinen(KOKO, KOKO);
        vihu.Color = Color.BloodRed;
        vihu.Shape = Shape.Circle;
        vihu.Mass = 1000;
        vihu.Tag = "vihu";
        vihu.Image = LoadImage("sika");
        vihu.CollisionIgnoreGroup = 3;
        vihu.Position = vihuSpawnit[RandomGen.NextInt(5)];
        vihu.LinearDamping = 1;
        vihu.Velocity = new Vector(-700, 0);
        Add(vihu);
        vihu.Removed += PisteLaskuriinLisays;
    }


    /// <summary>
    /// Kutsutaan kun vihollinen tuhotaan. Lisää pisteen.
    /// </summary>
    private void PisteLaskuriinLisays()
    {
        pisteLaskuri.AddValue(1);
    }
}
